package com.csc.library.process;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.Scanner;

import com.csc.library.cdi.GetCdiInstance;
import com.csc.library.entity.CompanyProfile;
import com.csc.library.entity.TrainingCourse;
import com.csc.library.entity.TrainingCourseManager;
import com.csc.library.entity.manager.CompanyManager;
import com.csc.library.status.StatusMessage;
import com.csc.library.training.utilitis.TimeUtils;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.MyLog;

public class ImportCourseManager {
	
	private CompanyManager cm = GetCdiInstance.of(CompanyManager.class);
	private CompanyProfile cpf ;
	
	private TrainingCourseManager tcm = GetCdiInstance.of(TrainingCourseManager.class);
	private TrainingCourse tc ;
	
	public ImportCourseManager(){

	}
	
	public boolean importFromFolder(String folder, String filetype){	
		return process(folder, filetype, "");
	}
	
	public boolean importFromFolder(String folder, String filetype, String backupfolder){	
		return process(folder, filetype, backupfolder);
	}
	
	private boolean process(String folder, String filetype, String backupfolder){
		try {
			
			File directory = new File(folder);
	        
			//get all the files from a directory
	        File[] fList = directory.listFiles();
	        String content = "";
	         
	        for (File file : fList){
	        	if (file.isFile() && file.getName().endsWith(filetype) && file.getName().startsWith("course_")) {
	        		try (BufferedReader br = new BufferedReader(new FileReader(file.getAbsolutePath()))) {

	        			while ((content = br.readLine()) != null) {
	        				System.out.println(content);
	        				if(content != null && !content.equals("")){
	        					save(content); /* raw data with '|' seperater column */
	        				}else{
	        					MyLog.debug(this, "!! data is empty !!");
	        				}
	        			}

	        		} catch (IOException e) {
	        			e.printStackTrace();
	        		}

	        		//backupFile(folder, file.getName());
	        	} 	        	
	        }        
	        
		}catch(Exception e){
			e.printStackTrace();
			return false ;
		}
		return true ;
	}
	
	
	private boolean backupFile(String folder, String filename){
		return backupFile(folder, filename, "");
	}
	
	private boolean backupFile(String folder, String filename, String target){
		try {
			CscCalendar today = new CscCalendar();
			File afile = new File(folder + "/" + filename);
			
			if(afile.renameTo(new File(folder + "/backup/" + filename + "_" + TimeUtils.getYYYYMMDDHHMMSS(today)))) {
				System.out.println("File is moved successful!");
	    		return true ;
	    	}else{
	    		System.out.println("File is failed to move!");
	    	}
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return false ;
	}
	
	private boolean save(String content){
		try {
			boolean m_complete = false ;
			boolean c_complete = false ;
			
			CourseImpProp eip = fetchRawData(content);
			
			if(eip != null){
				initInstances(eip);
				save(eip);	
			}
		}catch(Exception e){
			System.out.print(e.getMessage());
			return false ;
		}
		return false;
	}
	
	private void initInstances(CourseImpProp eip){
		try {
			cpf = cm.getCompanyByCompanyId(eip.getCompanyid());
			
		}catch(Exception e){
			MyLog.error(this, e);
		}
	}
	
	private boolean save(CourseImpProp cip){
		try {
			tc = tcm.createInstance(cip.getCompanyid()
					, cip.getTrainingid()
					, cip.getCourseid()
					, cip.getSessionid());
			
			tc.setTitle(cip.getTitle());
			tc.setLocation(cip.getLocation());
			tc.setStartdate(cip.getStartdate());
			tc.setStarttime(cip.getStarttime());
			tc.setEnddate(cip.getEnddate());
			tc.setEndtime(cip.getEndtime());
			tc.setTrainer(cip.getTrainer());
			tc.setDescription(cip.getDesc());
			tc.setStatus(cip.getStatus());
			tc.setObjective(cip.getObjective());
			tc.setReservstartdate(cip.getRstartdate());
			tc.setReservstarttime(cip.getRstarttime());
			tc.setReservenddate(cip.getRenddate());
			tc.setReservendtime(cip.getRendtime());
			tc.setPunchbefore(cip.getPunchbefore());
			tc.setPunchafter(cip.getPunchafter());
			tc.setPicture(cip.getPicture());
			tc.setSourcedownloadlink(cip.getSourcelink());
			tc.setVideolink(cip.getDownloadlink());
			
			
			
			StatusMessage sm = tcm.save(tc);
			MyLog.debug(this, "!! save emp complete : " + tc.getTrainingid() 
				+ "::" + tc.getCourseid() + "::" + tc.getSessionid());
			
			return true ;
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return false ;
	}
	
	/* convert raw data from file into EmployeeImpProp */
	private CourseImpProp fetchRawData(String content){
		try {
			return CourseImpProp.newInstance(content);
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return null ;
	}
	
	public boolean importWithFileName(String filename){
		try (Scanner scanner = new Scanner(new File(filename))) {

			while (scanner.hasNext()){
				System.out.println(scanner.nextLine());
			}

		} catch (Exception e) {
			e.printStackTrace();
			return false ;
		}
		return true ;
	}
	
}
