package com.csc.library.process;


import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.HashMap;
import java.util.List;

import com.csc.library.cdi.GetCdiInstance;
import com.csc.library.entity.CompanyEmployeeProfile;
import com.csc.library.entity.CompanyProfile;
import com.csc.library.entity.FileStore;
import com.csc.library.entity.Member;
import com.csc.library.entity.MemberProfile;
import com.csc.library.entity.MemberProfile.Sex;
import com.csc.library.entity.manager.CompanyEmployeeManager;
import com.csc.library.entity.manager.CompanyEmployeeManager.EmployeeWorkingManager;
import com.csc.library.entity.manager.CompanyManager;
import com.csc.library.entity.manager.MemberManager;
import com.csc.library.status.StatusMessage;
import com.csc.library.training.utilitis.TimeUtils;
import com.csc.library.utilities.CSCCheckUtility;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.FileUtils;
import com.csc.library.utilities.MyLog;
import com.csc.library.validator.EmailValidator;

import th.co.csc.apimodel.common.FaceTemplateCommonModel;
import th.co.csc.apimodel.request.ApplyCompanyRequestModel;
import th.co.csc.apimodel.request.RegisterNewEmployeeRequestModel;
import th.co.csc.apimodel.request.RegisterRequestModel;

public class ImportEmpManager {
	
	private static ImportEmpManager imp ;
	private String company ;
	private EmailValidator emailValidator = EmailValidator.getInstance();
	private CompanyManager cm = GetCdiInstance.of(CompanyManager.class);
	private EmployeeWorkingManager ewManager = GetCdiInstance.of(EmployeeWorkingManager.class);
	private CompanyProfile cpf ;
	
	private HashMap<String,String> hsBossList = new HashMap<String,String>();
	
	public ImportEmpManager(){
	}
	
	public static ImportEmpManager newInstance(){
		if(imp == null){
			imp = new ImportEmpManager() ;
		}
		return imp ;
	}
	
	public boolean importFromFolder(String folder, String filetype, String backupfolder){	
		return process(folder, filetype, backupfolder);
	}
	
	private boolean process(String folder, String filetype, String backupfolder){
		try {
			
			File directory = new File(folder);
	        
	        File[] fList = directory.listFiles();
	        String content = "";
	         
	        for (File file : fList){
	        	if (file.isFile() && file.getName().endsWith(filetype)) {
	        		//try (BufferedReader br = new BufferedReader(new FileReader(file.getAbsolutePath()))) {
	        		try (BufferedReader br = new BufferedReader(new InputStreamReader(new FileInputStream(file.getAbsolutePath()), "UTF-8"))){
	        			while ((content = br.readLine()) != null) {
	        				if(content != null && !content.equals("")){
	        					save(content); /* raw data with '|' seperater column */
	        				}else{
	        					MyLog.debug(this, "!! data is empty !!");
	        				}
	        			}

	        		} catch (IOException e) {
	        			e.printStackTrace();
	        		}
	        	   	  
	        		// set boss from legacy code 
	        		setZeeMeBossId();
	        		
	        		//back up
	        	    FileUtils.backupFile(folder, file.getName(), "");
	        	} 	        	
	        }        
	        
		}catch(Exception e){
			e.printStackTrace();
			return false ;
		}
		return true ;
	}
	
	private boolean setZeeMeBossId(){
		try {
			CompanyEmployeeManager cem = GetCdiInstance.of(CompanyEmployeeManager.class);
			
			if(company != null){
				
				List<CompanyEmployeeProfile> ls = ewManager.getView().findEmployeeProfileByCompanyId(company);
				
				ls.parallelStream().forEach(s -> {
					setBossId(s, cem);
					});
				
				return true ;
			}else{
				MyLog.error(this, "!! companyid is null, bossmajor is not be set !!");
			}
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return false ;
	}
	
	private boolean setBossId(CompanyEmployeeProfile cp, CompanyEmployeeManager cem){
		try {
			boolean needsave = false ;
			
			cp.setBossMajor(cpf.getOwnerId()); /*temporary*/
			needsave = true ;
			
			if(cp.getBossMajor() != null){
				/*
				if(cp.getBossMajor().length() < 20){
					cp.setBossMajor(findMemberIdByEmployeeCode(cp.getCompanyID(), cp.getBossMajor(), cem));
					needsave = true ;
				}
				
				if(cp.getBossMinor().length() < 20){
					cp.setBossMinor(findMemberIdByEmployeeCode(cp.getCompanyID(), cp.getBossMinor(), cem));
					needsave = true ;
				}*/
	
				if(needsave){
					ewManager.getAction().saveEmployeeProfile(cp);
				}
				
				return true ;
			}
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return false ;
	}
	
	private String findMemberIdByEmployeeCode(String comp, String code, CompanyEmployeeManager cem){
		try {
			CompanyEmployeeProfile cp = ewManager.getView().getEmployeeProfileByEmployeeCode(comp, code);
			if(cp != null){
				return cp.getMemberID();
			}
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return "";
	}
	
	private void initCompany(EmployeeImpProp eip){
		if(company == null) company = eip.getCompanyid();
	}
		
	private boolean save(String content){
		try {
			boolean m_complete = false ;
			boolean c_complete = false ;
			
			EmployeeImpProp eip = fetchRawData(content);
			
			if(eip != null){
				initCompany(eip);
				
				MemberProfile mb = getExistMember(eip);
				
				// temporay set default
				/*
				if(mb!=null && !CSCCheckUtility.isBlankOrNull(eip.getEmail()) && !emailValidator.isValid(eip.getEmail())){
					if(!mb.getEmailname().equals("")){
						eip.setEmail(mb.getEmailname());
					}
				}*/
				
				//create member when non exist 
				if((mb == null || mb.getMemberid().equals("")) && !CSCCheckUtility.isBlankOrNull(eip.getEmail()) 
						&& emailValidator.isValid(eip.getEmail())){
					mb = createMember(eip);		// create member 
					
					if(mb != null){
						m_complete = true ;
					}
				}else{
					if(!mb.getMemberid().equals("")){
						MyLog.debug(this, "!! member already exist !!");
					}else if(!CSCCheckUtility.isBlankOrNull(eip.getEmail())) {
						MyLog.debug(this , "!! invalid email, this member will not be create " + eip.getEmployeeid() + ":" + eip.getEmail());
					}else{
						MyLog.debug(this, "!! invalid data, member will not be create " + eip.getEmployeeid());
					}
				}
				
				// if found or gen member is success 
				if(mb != null){
					c_complete = createCompanyEmployee(eip, mb);	// create or update company employee
				}else{
					MyLog.debug(this, "!! save member incomplete, transaction rollback ");
				}
			}
			
			if(c_complete){
				return true;
			}
		}catch(Exception e){
			System.out.print(e.getMessage());
			return false ;
		}
		return false;
	}
	
	private MemberProfile getExistMember(EmployeeImpProp eip){
		try {
			MemberManager mm = GetCdiInstance.of(MemberManager.class);
			MemberProfile m = null;
			
			if(!eip.getPersonal_id().equals("")){
				m = mm.getMemberProfileByPeopleId(eip.getPersonal_id());
			}else if(!eip.getPassportid().equals("")){
				m = mm.getMemberProfileByPeopleId(eip.getPassportid());
			}
			
			if(m == null && !eip.getEmail().equals("")){
				m = mm.getMemberProfileByEmailname(eip.getEmail());
			}

			return m ;
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return null ;
	}
	
	/* convert raw data from file into EmployeeImpProp */
	private EmployeeImpProp fetchRawData(String content){
		return new EmployeeImpProp(content);
	}
	
	private ApplyCompanyRequestModel getRegisterRequestModel(EmployeeImpProp eip, MemberProfile mb){
		try {
			ApplyCompanyRequestModel rr = new ApplyCompanyRequestModel();
			rr.setCompanyId(eip.getCompanyid());
			rr.setBranchId(eip.getBranch());
			rr.setMemberId(mb.getMemberid());
			//rr.setEmployeeCode(eip.getEmployeeid()); // let system auto gen employeecode
			rr.setPositionName(eip.getPosition());
			rr.setApplyStatus("0");
			rr.setApplyDate(new CscCalendar().getDatetimeFormat());
			rr.setDepartmentName(eip.getDepartment());
			rr.setJointType(0);
			rr.setBossMajarId(eip.getBossid_1());
			
			return rr ;
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return null ;
	}
	
	private RegisterRequestModel getMemberRegisterModel(EmployeeImpProp eip, MemberProfile mb){
		try {
			RegisterRequestModel rr = new RegisterRequestModel();
			rr.setFirstName(eip.getFirstName());
			rr.setLastName(eip.getLastName());
			rr.setMobileNumber(eip.getTelephone());
			rr.setSex(Integer.parseInt(eip.getGender()));
			rr.setEmail(eip.getEmail());
			rr.setPassword(genPasswordWithBirthday(eip.getBirthday()));
			rr.setBirthDate(eip.getBirthday());
			rr.setIdCard(eip.getPersonal_id());
			rr.setPassportNo(eip.getPassportid());
			rr.setNationality(eip.getNationality());
			rr.setProfilePhoto(new byte[0]);
			rr.setDeviceName("import");
			
			/* template */
			FaceTemplateCommonModel fc = new FaceTemplateCommonModel();
			fc.setEnrollPhoto(new byte[0]);
			fc.setEyesTemplate1(new byte[0]);
			fc.setEyesTemplate2(new byte[0]);
			fc.setEyesTemplate3(new byte[0]);
			fc.setFacialTemplate1(new byte[0]);
			fc.setFacialTemplate2(new byte[0]);
			fc.setFacialTemplate3(new byte[0]);
			
			fc.setEnrollPhotoFileName("");
			fc.setFacialTemplate1FileName("");
			fc.setFacialTemplate2FileName("");
			fc.setFacialTemplate3FileName("");
			fc.setEyesTemplate1FileName("");
			fc.setEyesTemplate2FileName("");
			fc.setEyesTemplate3FileName("");
			rr.setFaceTemplate(fc);

			return rr ;
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return null ;
	}
	
	private boolean isCompEmpExist(EmployeeImpProp eip, MemberProfile mb){
		try {
			CompanyEmployeeManager cem = GetCdiInstance.of(CompanyEmployeeManager.class);
			CompanyEmployeeProfile cep = ewManager.getView().getEmployeeProfile(eip.getCompanyid(), mb.getMemberid(), eip.getEmployeeid());
			
			if(cep != null){
				return true ;
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return false ;
	}
	
	private boolean createCompanyEmployee(EmployeeImpProp eip, MemberProfile mb){
		try {
			//if(isCompEmpExist(eip, mb)){
			if(eip != null && !eip.getEmployeeid().equals("")){
				/* set company */
				/*
				if(company == null){
					company = ConnectorUtility.getCompCode(eip.getCompanyid());
				}*/
				
				if(cpf == null){
					cpf = cm.getCompanyProfileByCompanyId(eip.getCompanyid());
				}
				
				if(cpf != null){
					RegisterNewEmployeeRequestModel rnrm = new RegisterNewEmployeeRequestModel();
					rnrm.setApplyCompanyModel(getRegisterRequestModel(eip, mb));
					rnrm.setMemberRegisterModel(getMemberRegisterModel(eip, mb));
					
					GenerateCompanyEmployee gce = new GenerateCompanyEmployee();
					gce.process(rnrm,eip,mb,cpf);
					
					CompanyEmployeeProfile cp = ewManager.getView().getEmployeeProfile(eip.getCompanyid(), mb.getMemberid());
					cp.setLagacyCode(eip.getEmployeeid());
					ewManager.getAction().saveEmployeeProfile(cp);
					
					return true ;
				}else{
					MyLog.debug(this, "!! company not found, employee will not be insert to companyemployee");
				}
			}else{
				MyLog.debug(this, "!! employee data is blank ");
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return false ;
	}
	
	private Member createMember(EmployeeImpProp eip){
		try {
			MemberManager mbm = GetCdiInstance.of(MemberManager.class);
			Member mp = mbm.createMemberInstance();
						
			mp.setFirstname(eip.getFirstName());
			mp.setLastname(eip.getLastName());
			mp.setIdpeople(eip.getPersonal_id());
			mp.setPassportno(eip.getPassportid());
			mp.setBirthday(TimeUtils.getYYYYMMDD(eip.getBirthday()));
			mp.setEmailname(eip.getEmail());
			mp.setNationality(eip.getNationality());
			mp.setSex(Sex.getSex(eip.convertGender()));
			mp.setTelnumber(eip.getTelephone());
			mp.setPassword(genPassword(eip));
			
			FileStore fs = GetCdiInstance.of(FileStore.class);
			mp.setFileStoreEyetemplate1(fs);
			mp.setFileStoreEyetemplate2(fs);
			mp.setFileStoreEyetemplate3(fs);
			mp.setFileStoreFacialtemplate1(fs);
			mp.setFileStoreFacialtemplate2(fs);
			mp.setFileStoreFacialtemplate1(fs);
			
			StatusMessage sm = mbm.saveMember(mp);
			
			return mp ;
		}catch(Exception e){
			MyLog.error(this, e);
		}
		MyLog.debug(this, "!! could not create member : " + eip.getEmployeeid()) ;
		return null ;
	}
	
	private String genPassword(EmployeeImpProp eip){
		try {
			GenPassword gp = new GenPassword();
			return gp.genPassword(eip);
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return eip.getPersonal_id() ; 
	}
	
	private boolean isBossCodeExist(String boss){
		try {
			if(hsBossList.containsKey(boss)){
				return true ;
			}
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return false ;
	}
	
	private String getBossCode(String boss){
		return hsBossList.get(boss);
	}
	
	private String genPasswordWithBirthday(String birthdate){
		String DASH = "-" ;
		String SLASH = "/" ;
		try {
			String tt[] = {"1234","",""} ;
			if(birthdate.indexOf(DASH) > 0) {
				tt = birthdate.split(DASH) ;
			}else if(birthdate.indexOf(DASH) > 0) {
				tt = birthdate.split(SLASH) ;
			}
			return tt[2] + tt[1] + tt[0] ; // DDMMYYYY 
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return "1234" ;
	}
}
