package com.csc.library.process;


import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.OutputStream;

import com.csc.library.ftp.ConnectProp;
import com.csc.library.ftp.FTPManager;
import com.csc.library.training.utilitis.TimeUtils;
import com.csc.library.utilities.FileUtils;
import com.csc.library.utilities.MyLog;
import com.jcraft.jsch.Channel;
import com.jcraft.jsch.ChannelSftp;
import com.jcraft.jsch.JSch;
import com.jcraft.jsch.Session;

public class UpdateFromTRDParty {

	public boolean loadFile(ConnectProp cnp){
		try {
			if(cnp.getProtocol().equals("FTP")){
				return loadFileFTP(cnp);
			}else if(cnp.getProtocol().equals("SFTP")){
				return loadFileSFTP(cnp);
			}
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return false ;
	}
	
	public boolean loadFileFTP(ConnectProp cn){
		try {
			FTPManager ftm = new FTPManager();
			if(!ftm.downloadAll(cn, cn.getTarget(), "")){
				System.out.print("error : download failed");
				return false ;
			}
		}catch(Exception e){
			e.printStackTrace();
			return false ;
		}
		return true ;
	}
	
	public boolean loadFileSFTP(ConnectProp cn){
		Session session = null;
		Channel channel = null;
		ChannelSftp channelSftp = null;
		
		BufferedInputStream bis = null;
		OutputStream os = null;
		BufferedOutputStream bos = null;

		try {
			String SFTPHOST = cn.getAddress();
			int SFTPPORT = Integer.parseInt(cn.getPort());
			String SFTPUSER = cn.getUsername();
			String SFTPPASS = cn.getPassword();
			String SFTPWORKINGDIR = cn.getDirectory();
			
			JSch jsch = new JSch();
			session = jsch.getSession(SFTPUSER, SFTPHOST, SFTPPORT);
			session.setPassword(SFTPPASS);
			
			java.util.Properties config = new java.util.Properties();
			config.put("StrictHostKeyChecking", "no");
			session.setConfig(config);
			session.connect();
			
			channel = session.openChannel("sftp");
			channel.connect();
			channelSftp = (ChannelSftp) channel;
			channelSftp.cd(SFTPWORKINGDIR);
			byte[] buffer = new byte[1024];
			
			bis = new BufferedInputStream(channelSftp.get(cn.getFilename()));
			
			/*target main folder*/
			if(FileUtils.folderExist(cn.getTargetFolder())){
				if(FileUtils.folderExist(cn.getTarget())){
					File newFile = new File(cn.getTarget() + "/" + cn.getFilename());
					os = new FileOutputStream(newFile);
					bos = new BufferedOutputStream(os);
					int readCount;
					
					while ((readCount = bis.read(buffer)) > 0) {
						bos.write(buffer, 0, readCount);
					}
				}
			}
			
			MyLog.debug(this, "! Load file successful !");
			return true ;
		} catch (Exception ex) {
			ex.printStackTrace();
		}finally{
			try {
				bis.close();
				bos.close();
			}catch(Exception e){
				MyLog.error(this, e);
			}
		}
		return false ;
	}
	
	

}
