package com.csc.library.training.controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;

import com.csc.library.cdi.GetCdiInstance;
import com.csc.library.entity.CompanyEmployeeProfile;
import com.csc.library.entity.TrainingCourse;
import com.csc.library.entity.TrainingCourseManager;
import com.csc.library.status.StatusMessage;
import com.csc.library.utilities.ConnectorUtility;
import com.csc.library.utilities.FileUtils;
import com.csc.library.utilities.MyLog;

public class ImportCourse {
	
	private TrainingCourseManager cem = GetCdiInstance.of(TrainingCourseManager.class);
	
	public static ImportCourse newInstance(){
		return new ImportCourse();
	}
	
	public boolean importFromFolder(String folder, String filetype, String backupfolder){
		try {
			File directory = new File(folder);
	        
	        File[] fList = directory.listFiles();
	        String content = "";
	         
	        for (File file : fList){
	        	if (file.isFile() && file.getName().endsWith(filetype)) {
	        		try (BufferedReader br = new BufferedReader(new FileReader(file.getAbsolutePath()))) {

	        			while ((content = br.readLine()) != null) {
	        				System.out.println(content);
	        				if(content != null && !content.equals("")){
	        					save(content); /* raw data with '|' seperater column */
	        				}else{
	        					MyLog.debug(this, "!! data is empty !!");
	        				}
	        			}

	        		} catch (IOException e) {
	        			e.printStackTrace();
	        		}
	        	   	  
	        	    FileUtils.backupFile(folder, file.getName(), "");
	        	} 	        	
	        }        
	        
		}catch(Exception e){
			e.printStackTrace();
			return false ;
		}
		return true ;
	}

	@SuppressWarnings("unused")
	private boolean save(String content){
		try {
			TrainingProp tp = fetchRawData(content);
			
			if(createCourse(tp)){ /* create or update company employee */
				return true;
			}
		}catch(Exception e){
			System.out.print(e.getMessage());
		}
		return false;
	}
	
	/* convert raw data from file into EmployeeImpProp */
	private TrainingProp fetchRawData(String content){
		try {
			return TrainingProp.newInstance(content);
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return null ;
	}
	
	private boolean createCourse(TrainingProp trc){
		try {
			
			if(trc != null && !trc.getTrainingid().equals("")){
				
				 TrainingCourse ce = cem.createInstance(ConnectorUtility.getCompCode(trc.getCompanyid())
						,trc.getTrainingid()
						,trc.getCourseid()
						,trc.getSessid());
				
				ce.setTitle(trc.getTitle());
				ce.setLocation(trc.getLocation());
				ce.setStartdate(trc.getStartdate());
				ce.setStarttime(trc.getStarttime());
				ce.setEnddate(trc.getEnddate());
				ce.setEndtime(trc.getEndtime());
				ce.setTrainer(trc.getTrainer());
				ce.setDescription(trc.getDesc());
				ce.setStatus(trc.getStatus());
				ce.setObjective(trc.getObjective());
				ce.setReservstartdate(trc.getRstartdate());
				ce.setReservstarttime(trc.getStarttime());
				ce.setReservenddate(trc.getRenddate());
				ce.setReservendtime(trc.getRendtime());
				ce.setPunchbefore(trc.getPunchbefore());
				ce.setPunchafter(trc.getPunchafter());
				ce.setPicture(trc.getPicture());
				ce.setSourcedownloadlink(trc.getSourcelink());
				ce.setVideolink(trc.getVideolink());
				
				StatusMessage sm = cem.save(ce);
				
				return true ;
			}else{
				MyLog.debug(this, "!! employee data is blank ");
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return false ;
	}
	
	private HashMap<String, String> createMapping(List<CompanyEmployeeProfile> cp){
		HashMap<String, String> hm = new HashMap<String, String>();
		try {
			for(CompanyEmployeeProfile i: cp){
				hm.put(i.getEmployeeCode(), i.getMemberID());
			}
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return hm ;
	}
	
	private String findMemberIdByEmployeeCode(String code, HashMap<String,String> emplist){
		try {
			if(emplist.get(code) != null){
				return emplist.get(code);
			}
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return "";
	}
}
