package com.csc.library.training.controller;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;

import com.csc.library.cdi.GetCdiInstance;
import com.csc.library.entity.CompanyEmployeeProfile;
import com.csc.library.entity.Trainee;
import com.csc.library.entity.TraineeManager;
import com.csc.library.entity.TrainingCourse;
import com.csc.library.entity.TrainingCourseManager;
import com.csc.library.entity.manager.CompanyEmployeeManager.EmployeeWorkingManager;
import com.csc.library.status.StatusMessage;
import com.csc.library.training.utilitis.TimeUtils;
import com.csc.library.utilities.ConnectorUtility;
import com.csc.library.utilities.CscCalendar;
import com.csc.library.utilities.FileUtils;
import com.csc.library.utilities.MyLog;

public class ImportEmpCourse {
	
	private TraineeManager tm = GetCdiInstance.of(TraineeManager.class);
	//private HashMap<String, TrainingCourse> hmCourse ;
	private HashMap<String, MemberMapping> hmMember ; /*temporary*/
	
	public static ImportEmpCourse newInstance(){
		return new ImportEmpCourse();
	}
	
	public boolean importFromFolder(String folder, String filetype, String backupfolder){
		try {
			File directory = new File(folder);
	        
	        File[] fList = directory.listFiles();
	        String content = "";
	         
	        for (File file : fList){
	        	if (file.isFile() && file.getName().endsWith(filetype)) {
	        		try (BufferedReader br = new BufferedReader(new FileReader(file.getAbsolutePath()))) {

	        			while ((content = br.readLine()) != null) {
	        				System.out.println(content);
	        				if(content != null && !content.equals("")){
	        					save(content); /* raw data with '|' seperater column */
	        				}else{
	        					MyLog.debug(this, "!! data is empty !!");
	        				}
	        			}

	        		} catch (IOException e) {
	        			e.printStackTrace();
	        		}
	        	   	  
	        	    FileUtils.backupFile(folder, file.getName(), "");
	        	} 	        	
	        }        
	        
		}catch(Exception e){
			e.printStackTrace();
			return false ;
		}
		return true ;
	}
	
	@SuppressWarnings("unused")
	private boolean save(String content){
		try {
			EmpCourseProp tp = fetchRawData(content);
			
			if(createEmpCourse(tp)){ /* create or update employee course */
				return true;
			}
		}catch(Exception e){
			System.out.print(e.getMessage());
			return false ;
		}
		return false;
	}
	
	/* convert raw data from file into EmployeeImpProp */
	private EmpCourseProp fetchRawData(String content){
		try {
			return EmpCourseProp.newInstance(content);
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return null ;
	}
	
	private MemberMapping getMemberByLegacyCode(String comp,String lg){
		try {
			/* temporary */
			if(hmMember == null){
				hmMember = new HashMap<String, MemberMapping>();
				
				EmployeeWorkingManager ewManager = GetCdiInstance.of(EmployeeWorkingManager.class);
				List<CompanyEmployeeProfile> lce = ewManager.getView().findEmployeeProfileByCompanyId(comp);
				
				for(CompanyEmployeeProfile cp: lce){
					hmMember.put(cp.getLagacyCode(), new MemberMapping(cp));
				}
			}
			
			if(hmMember.containsKey(lg)){
				return hmMember.get(lg);
			}
			/* end temporary */
			
			/* for production  */
			/* need method from CompanyEmployeeManager to be called */
			
		}catch(Exception e){
			MyLog.error(this, e);
		}
		return null ;
	}
	
	private boolean createEmpCourse(EmpCourseProp trc){
		try {
			CscCalendar now = new CscCalendar();
			
			if(trc != null && !trc.getTrainingid().equals("")){
				MemberMapping mp = getMemberByLegacyCode(trc.getCompanyid(), trc.getLegacycode());
				
				if(mp != null){
					
					Trainee ce = tm.createInstance(ConnectorUtility.getCompCode(trc.getCompanyid())
							,mp.getMemberid()
							,ImportEmpCourse
								.CourseFindingHandling.newInstance()
								.getCourse(trc).getStartdate()
							,trc.getTrainingid()
							,trc.getCourseid()
							,trc.getSessid());
					
					ce.setStarttime(ImportEmpCourse
								.CourseFindingHandling.newInstance()
								.getCourse(trc).getStarttime());
					ce.setEmployeeid(mp.getEmployeecode());
					ce.setLegacycode(mp.getLegacycode());
					ce.setStatus(Trainee.Status.Unread.getStatus());
					ce.setEditby("ImportEmpCourse");
					ce.setEditdate(TimeUtils.getYYYYMMDD(now));
					ce.setEdittime(TimeUtils.getHHMMSS(now));
					
					StatusMessage sm = tm.insert(ce);
					
					sendNotification(ce
							, ImportEmpCourse
								.CourseFindingHandling.newInstance()
								.getCourse(trc));/* send notification */
					return true ;
				}
			}else{
				MyLog.debug(this, "!! search member by legacy code not found ");
			}
		}catch(Exception e){
			e.printStackTrace();
		}
		return false ;
	}
	
	/* send notification to mobile */
	private void sendNotification(Trainee tn, TrainingCourse tc){
		SendNewCourseNotification.newInstance().send(tn, tc);
	}
	
	private static class CourseFindingHandling {
		private static HashMap<String, TrainingCourse> hmCourse ;
		private static CourseFindingHandling cf = null ;

		private TrainingCourseManager tcm = GetCdiInstance.of(TrainingCourseManager.class);

		public static CourseFindingHandling newInstance(){
			if(cf == null){
				cf = new CourseFindingHandling();
			}
			
			if( hmCourse == null) {
				hmCourse = new HashMap<String, TrainingCourse>();
			}
			return cf ; 
		}
		
		public TrainingCourse getCourse(EmpCourseProp ecp){
			try {
				if(hmCourse.containsKey(genKey(ecp))){
					return hmCourse.get(genKey(ecp));
				}else{
					TrainingCourse tc = tcm.getCourse(ecp.getCompanyid()
							, ecp.getTrainingid()
							, ecp.getCourseid()
							, ecp.getSessid());
					
					hmCourse.put(genKey(ecp), tc);
					
					return tc ;
				}
			}catch(Exception e){
				MyLog.error(this, e);
			}
			return null ;
		}
		
		private String genKey(EmpCourseProp ecp){
			return ecp.getTrainingid() + ":" + ecp.getCourseid() + ":" + ecp.getSessid();
		}
	}
	
	private class MemberMapping {
		private String memberid ;
		private String employeecode ;
		private String legacycode ;
		
		public MemberMapping(CompanyEmployeeProfile cep){
			this.setMemberid(cep.getMemberID());
			this.setEmployeecode(cep.getEmployeeCode());
			this.setLegacycode(cep.getLagacyCode());
		}
		
		public String getMemberid() {
			return memberid;
		}
		public void setMemberid(String memberid) {
			this.memberid = memberid;
		}
		public String getEmployeecode() {
			return employeecode;
		}
		public void setEmployeecode(String employeecode) {
			this.employeecode = employeecode;
		}
		public String getLegacycode() {
			return legacycode;
		}
		public void setLegacycode(String legacycode) {
			this.legacycode = legacycode;
		}
		
		
	}

}
